@@ -28,11 +28,13 @@ import pprint
 import re
 import urllib2
 import urlparse
+import select
+import statsd
 import shutil
+import socket
 import string
 import git
 
-import zuul
 import zuul.scheduler
 import zuul.launcher.jenkins
 import zuul.trigger.gerrit
@@ -72,6 +74,10 @@ def init_repo(project):
     path = os.path.join(UPSTREAM_ROOT, project)
     repo = git.Repo.init(path)
 
+    repo.config_writer().set_value('user', 'email', 'user@example.com')
+    repo.config_writer().set_value('user', 'name', 'User Name')
+    repo.config_writer().write()
+
     fn = os.path.join(path, 'README')
     f = open(fn, 'w')
     f.write("test\n")
@@ -170,6 +176,7 @@ class FakeChange(object):
         self.depends_on_change = None
         self.needed_by_changes = []
         self.fail_merge = False
+        self.messages = []
         self.data = {
             'branch': branch,
             'comments': [],
@@ -235,6 +242,19 @@ class FakeChange(object):
                  "uploader": {"name": "User Name"}}
         return event
 
+    def getChangeRestoredEvent(self):
+        event = {"type": "change-restored",
+                 "change": {"project": self.project,
+                            "branch": self.branch,
+                            "id": "I5459869c07352a31bfb1e7a8cac379cabfcb25af",
+                            "number": str(self.number),
+                            "subject": self.subject,
+                            "owner": {"name": "User Name"},
+                            "url": "https://hostname/3"},
+                 "restorer": {"name": "User Name"},
+                 "reason": ""}
+        return event
+
     def addApproval(self, category, value):
         approval = {'description': self.categories[category][0],
                     'type': category,
@@ -366,6 +386,7 @@ class FakeGerrit(object):
     def review(self, project, changeid, message, action):
         number, ps = changeid.split(',')
         change = self.changes[int(number)]
+        change.messages.append(message)
         if 'submit' in action:
             change.setMerged()
         if message:
@@ -642,6 +663,32 @@ class FakeGerritTrigger(zuul.trigger.gerrit.Gerrit):
         return os.path.join(UPSTREAM_ROOT, project.name)
 
 
+class FakeStatsd(threading.Thread):
+    def __init__(self):
+        threading.Thread.__init__(self)
+        self.sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
+        self.sock.bind(('', 0))
+        self.port = self.sock.getsockname()[1]
+        self.wake_read, self.wake_write = os.pipe()
+        self.stats = []
+
+    def run(self):
+        while True:
+            read_ready = select.select([self.sock, self.wake_read], [], [])[0]
+            for sock in read_ready:
+                if sock is self.sock:
+                    data = self.sock.recvfrom(1024)
+                    if not data:
+                        return
+                    self.stats.append(data[0])
+                else:
+                    # wake_read
+                    return
+
+    def stop(self):
+        os.write(self.wake_write, '1\n')
+
+
 class testScheduler(unittest.TestCase):
     log = logging.getLogger("zuul.test")
 
@@ -659,7 +706,17 @@ class testScheduler(unittest.TestCase):
         init_repo("org/project3")
         init_repo("org/one-job-project")
         init_repo("org/nonvoting-project")
+        init_repo("org/templated-project")
         self.config = CONFIG
+
+        self.statsd = FakeStatsd()
+        os.environ['STATSD_HOST'] = 'localhost'
+        os.environ['STATSD_PORT'] = str(self.statsd.port)
+        self.statsd.start()
+        # the statsd client object is configured in the statsd module import
+        reload(statsd)
+        reload(zuul.scheduler)
+
         self.sched = zuul.scheduler.Scheduler()
 
         def jenkinsFactory(*args, **kw):
@@ -699,6 +756,8 @@ class testScheduler(unittest.TestCase):
         self.gerrit.stop()
         self.sched.stop()
         self.sched.join()
+        self.statsd.stop()
+        self.statsd.join()
         #shutil.rmtree(TEST_ROOT)
 
     def waitUntilSettled(self):
@@ -745,6 +804,21 @@ class testScheduler(unittest.TestCase):
                     print 'heads', queue.severed_heads
                 assert len(queue.severed_heads) == 0
 
+    def assertReportedStat(self, key, value=None):
+        start = time.time()
+        while time.time() < (start + 5):
+            for stat in self.statsd.stats:
+                k, v = stat.split(':')
+                if key == k:
+                    if value is None:
+                        return
+                    if value == v:
+                        return
+            time.sleep(0.1)
+
+        pprint.pprint(self.statsd.stats)
+        raise Exception("Key %s not found in reported stats" % key)
+
     def test_jobs_launched(self):
         "Test that jobs are launched and a change is merged"
         A = self.fake_gerrit.addFakeChange('org/project', 'master', 'A')
@@ -763,6 +837,44 @@ class testScheduler(unittest.TestCase):
         assert A.reported == 2
         self.assertEmptyQueues()
 
+        self.assertReportedStat('gerrit.event.comment-added', '1|c')
+        self.assertReportedStat('zuul.pipeline.gate.current_changes', '1|g')
+        self.assertReportedStat('zuul.job.project-merge')
+        self.assertReportedStat('zuul.pipeline.gate.resident_time')
+        self.assertReportedStat('zuul.pipeline.gate.total_changes', '1|c')
+        self.assertReportedStat(
+            'zuul.pipeline.gate.org.project.resident_time')
+        self.assertReportedStat(
+            'zuul.pipeline.gate.org.project.total_changes', '1|c')
+
+    def test_duplicate_pipelines(self):
+        "Test that a change matching multiple pipelines works"
+        A = self.fake_gerrit.addFakeChange('org/project', 'master', 'A')
+        self.fake_gerrit.addEvent(A.getChangeRestoredEvent())
+        self.waitUntilSettled()
+        jobs = self.fake_jenkins.job_history
+
+        print jobs
+        print A.messages
+
+        self.assertEmptyQueues()
+
+        assert len(jobs) == 2
+        jobs[0].name == 'project-test1'
+        jobs[1].name == 'project-test1'
+
+        assert len(A.messages) == 2
+        if 'dup1/project-test1' in A.messages[0]:
+            assert 'dup1/project-test1' in A.messages[0]
+            assert 'dup2/project-test1' not in A.messages[0]
+            assert 'dup1/project-test1' not in A.messages[1]
+            assert 'dup2/project-test1' in A.messages[1]
+        else:
+            assert 'dup1/project-test1' in A.messages[1]
+            assert 'dup2/project-test1' not in A.messages[1]
+            assert 'dup1/project-test1' not in A.messages[0]
+            assert 'dup2/project-test1' in A.messages[0]
+
     def test_parallel_changes(self):
         "Test that changes are tested in parallel and merged in series"
         self.fake_jenkins.hold_jobs_in_build = True
@@ -1310,6 +1422,20 @@ class testScheduler(unittest.TestCase):
         assert B.reported == 2
         self.assertEmptyQueues()
 
+    def test_job_from_templates_launched(self):
+        "Test whether a job generated via a template can be launched"
+        A = self.fake_gerrit.addFakeChange(
+            'org/templated-project', 'master', 'A')
+        self.fake_gerrit.addEvent(A.getPatchsetCreatedEvent(1))
+        self.waitUntilSettled()
+        jobs = self.fake_jenkins.job_history
+        job_names = [x.name for x in jobs]
+
+        assert 'project-test1' in job_names
+        assert 'project-test2' in job_names
+        assert jobs[0].result == 'SUCCESS'
+        assert jobs[1].result == 'SUCCESS'
+
     def test_dependent_changes_dequeue(self):
         "Test that dependent patches are not needlessly tested"
         A = self.fake_gerrit.addFakeChange('org/project', 'master', 'A')
@@ -1970,6 +2096,17 @@ class testScheduler(unittest.TestCase):
         assert D.reported == 2
         self.assertEmptyQueues()
 
+    def test_statsd(self):
+        "Test each of the statsd methods used in the scheduler"
+        import extras
+        statsd = extras.try_import('statsd.statsd')
+        statsd.incr('test-incr')
+        statsd.timing('test-timing', 3)
+        statsd.gauge('test-guage', 12)
+        self.assertReportedStat('test-incr', '1|c')
+        self.assertReportedStat('test-timing', '3|ms')
+        self.assertReportedStat('test-guage', '12|g')
+
     def test_file_jobs(self):
         "Test that file jobs run only when appropriate"
         A = self.fake_gerrit.addFakeChange('org/project', 'master', 'A')
@@ -1994,3 +2131,8 @@ class testScheduler(unittest.TestCase):
         assert B.data['status'] == 'MERGED'
         assert B.reported == 2
         self.assertEmptyQueues()
+
+    def test_test_config(self):
+        "Test that we can test the config"
+        sched = zuul.scheduler.Scheduler()
+        sched.testConfig(CONFIG.get('zuul', 'layout_config'))
