@@ -23,7 +23,6 @@ import threading
 import time
 import urllib   # for extending jenkins lib
 import urllib2  # for extending jenkins lib
-import urlparse
 from uuid import uuid4
 
 import jenkins
@@ -378,21 +377,6 @@ class Jenkins(object):
             self.log.error("Build %s has not started but "
                            "was not found in queue" % build)
 
-    def getBestBuildURL(self, url):
-        try:
-            test_url = urlparse.urljoin(url, 'testReport')
-            self.jenkins.jenkins_open(urllib2.Request(test_url))
-            return test_url
-        except:
-            pass
-        try:
-            console_url = urlparse.urljoin(url, 'consoleFull')
-            self.jenkins.jenkins_open(urllib2.Request(console_url))
-            return console_url
-        except:
-            pass
-        return url
-
     def setBuildDescription(self, build, description):
         if not build.number:
             return
@@ -412,8 +396,6 @@ class Jenkins(object):
             self.log.debug("Found build %s" % build)
             del self.builds[uuid]
             if url:
-                build.base_url = url
-                url = self.getBestBuildURL(url)
                 build.url = url
             build.result = status
             build.number = number
