@@ -1,5 +1,7 @@
 # Copyright 2012 Hewlett-Packard Development Company, L.P.
 # Copyright 2013 OpenStack Foundation
+# Copyright 2013 Antoine "hashar" Musso
+# Copyright 2013 Wikimedia Foundation Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -32,6 +34,28 @@ import merger
 statsd = extras.try_import('statsd.statsd')
 
 
+def deep_format(obj, paramdict):
+    """Apply the paramdict via str.format() to all string objects found within
+       the supplied obj. Lists and dicts are traversed recursively.
+
+       Borrowed from Jenkins Job Builder project"""
+    if isinstance(obj, str):
+        ret = obj.format(**paramdict)
+    elif isinstance(obj, list):
+        ret = []
+        for item in obj:
+            ret.append(deep_format(item, paramdict))
+    elif isinstance(obj, dict):
+        ret = {}
+        for item in obj:
+            exp_item = item.format(**paramdict)
+
+            ret[exp_item] = deep_format(obj[item], paramdict)
+    else:
+        ret = obj
+    return ret
+
+
 class Scheduler(threading.Thread):
     log = logging.getLogger("zuul.Scheduler")
 
@@ -45,6 +69,7 @@ class Scheduler(threading.Thread):
         self._stopped = False
         self.launcher = None
         self.trigger = None
+        self.config = None
 
         self.trigger_event_queue = Queue.Queue()
         self.result_event_queue = Queue.Queue()
@@ -54,6 +79,7 @@ class Scheduler(threading.Thread):
         self.pipelines = {}
         self.jobs = {}
         self.projects = {}
+        self.project_templates = {}
         self.metajobs = {}
 
     def stop(self):
@@ -125,6 +151,16 @@ class Scheduler(threading.Thread):
                                 toList(trigger.get('email_filter')))
                 manager.event_filters.append(f)
 
+        for project_template in data.get('project-templates', []):
+            # Make sure the template only contains valid pipelines
+            tpl = dict(
+                (pipe_name, project_template.get(pipe_name))
+                for pipe_name in self.pipelines.keys()
+                if pipe_name in project_template
+            )
+            self.project_templates[project_template.get('name')] \
+                = tpl
+
         for config_job in data.get('jobs', []):
             job = self.getJob(config_job['name'])
             # Be careful to only set attributes explicitly present on
@@ -176,6 +212,17 @@ class Scheduler(threading.Thread):
 
         for config_project in data.get('projects', []):
             project = Project(config_project['name'])
+
+            for requested_template in config_project.get('template', []):
+                # Fetch the template from 'project-templates'
+                tpl = self.project_templates.get(
+                    requested_template.get('name'))
+                # Expand it with the project context
+                expanded = deep_format(tpl, requested_template)
+                # Finally merge the expansion with whatever has been already
+                # defined for this project
+                config_project.update(expanded)
+
             self.projects[config_project['name']] = project
             mode = config_project.get('merge-mode')
             if mode and mode == 'cherry-pick':
@@ -199,6 +246,16 @@ class Scheduler(threading.Thread):
         else:
             merge_root = '/var/lib/zuul/git'
 
+        if self.config.has_option('zuul', 'git_user_email'):
+            merge_email = self.config.get('zuul', 'git_user_email')
+        else:
+            merge_email = None
+
+        if self.config.has_option('zuul', 'git_user_name'):
+            merge_name = self.config.get('zuul', 'git_user_name')
+        else:
+            merge_name = None
+
         if self.config.has_option('zuul', 'push_change_refs'):
             push_refs = self.config.getboolean('zuul', 'push_change_refs')
         else:
@@ -210,7 +267,7 @@ class Scheduler(threading.Thread):
             sshkey = None
 
         self.merger = merger.Merger(self.trigger, merge_root, push_refs,
-                                    sshkey)
+                                    sshkey, merge_email, merge_name)
         for project in self.projects.values():
             url = self.trigger.getGitUrl(project)
             self.merger.addProject(project, url)
@@ -508,6 +565,12 @@ class BasePipelineManager(object):
         self.success_action = {}
         self.failure_action = {}
         self.start_action = {}
+        if self.sched.config and self.sched.config.has_option(
+            'zuul', 'report_times'):
+            self.report_times = self.sched.config.getboolean(
+                'zuul', 'report_times')
+        else:
+            self.report_times = True
 
     def __str__(self):
         return "<%s %s>" % (self.__class__.__name__, self.pipeline.name)
@@ -869,22 +932,30 @@ class BasePipelineManager(object):
                         result = job.failure_message
                     if job.failure_pattern:
                         pattern = job.failure_pattern
-                url = None
-                if build.url:
-                    if pattern:
-                        url = pattern.format(change=changeish,
-                                             pipeline=self.pipeline,
-                                             job=job,
-                                             build=build)
-                    else:
-                        url = build.url
-                if not url:
-                    url = job.name
+                if pattern:
+                    url = pattern.format(change=changeish,
+                                         pipeline=self.pipeline,
+                                         job=job,
+                                         build=build)
+                else:
+                    url = build.url or job.name
                 if not job.voting:
                     voting = ' (non-voting)'
                 else:
                     voting = ''
-                ret += '- %s : %s%s\n' % (url, result, voting)
+                if self.report_times and build.end_time and build.start_time:
+                    dt = int(build.end_time - build.start_time)
+                    m, s = divmod(dt, 60)
+                    h, m = divmod(m, 60)
+                    if h:
+                        elapsed = ' in %dh %02dm %02ds' % (h, m, s)
+                    elif m:
+                        elapsed = ' in %dm %02ds' % (m, s)
+                    else:
+                        elapsed = ' in %ds' % (s)
+                else:
+                    elapsed = ''
+                ret += '- %s : %s%s%s\n' % (url, result, elapsed, voting)
         return ret
 
     def formatDescription(self, build):
@@ -900,10 +971,10 @@ class BasePipelineManager(object):
         change = build.build_set.change
 
         for build in build.build_set.getBuilds():
-            if build.base_url:
+            if build.url:
                 concurrent_builds += """\
 <li>
-  <a href="{build.base_url}">
+  <a href="{build.url}">
   {build.job.name} #{build.number}</a>: {build.result}
 </li>
 """.format(build=build)
@@ -919,7 +990,7 @@ class BasePipelineManager(object):
             if other_build:
                 other_builds += """\
 <li>
-  Preceded by: <a href="{build.base_url}">
+  Preceded by: <a href="{build.url}">
   {build.job.name} #{build.number}</a>
 </li>
 """.format(build=other_build)
@@ -930,7 +1001,7 @@ class BasePipelineManager(object):
             if other_build:
                 other_builds += """\
 <li>
-  Succeeded by: <a href="{build.base_url}">
+  Succeeded by: <a href="{build.url}">
   {build.job.name} #{build.number}</a>
 </li>
 """.format(build=other_build)
